use actix_web::{http::header::LOCATION, web, HttpResponse, Responder};
use garde::Validate;
use sqlx::PgPool;

use crate::{
    endpoints::{availability::post_new::AvailabillityForm, IdPath},
    models::{Availability, AvailabilityChangeset, AvailabilityContext, AvailabilityTime, User},
    utils::{self, ApplicationError},
};

#[actix_web::post("/availabillity/edit/{id}")]
pub async fn post(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    path: web::Path<IdPath>,
    form: web::Form<AvailabillityForm>,
) -> Result<impl Responder, ApplicationError> {
    let Some(availabillity) = Availability::read_by_id(pool.get_ref(), path.id).await? else {
        return Ok(HttpResponse::NotFound().finish());
    };

    if availabillity.user_id != user.id {
        return Err(ApplicationError::Unauthorized);
    }

    let existing_availabilities =
        Availability::read_by_user_and_date(pool.get_ref(), user.id, &availabillity.date).await?;
    let context = AvailabilityContext {
        existing_availabilities,
    };

    let time = if form.from.is_some() && form.till.is_some() {
        AvailabilityTime::Temporarily(form.from.unwrap(), form.till.unwrap())
    } else {
        AvailabilityTime::WholeDay
    };

    let changeset = AvailabilityChangeset {
        time,
        comment: form.comment.clone(),
    };

    if let Err(e) = changeset.validate_with(&context) {
        return Ok(HttpResponse::BadRequest().body(e.to_string()));
    };

    // TODO: check if adjacent availabillity is present and combine
    Availability::update(pool.get_ref(), availabillity.id, changeset).await?;

    let url = utils::get_return_url_for_date(&form.date);
    Ok(HttpResponse::Found()
        .insert_header((LOCATION, url.clone()))
        .insert_header(("HX-LOCATION", url))
        .finish())
}
